package config

import (
	"bytes"
	"github.com/rs/zerolog"
	"github.com/spf13/viper"
	"io/ioutil"
	//	"fmt"
	"os"
)

const (
	Version = "0.0"
	Title   = "jupemail"
)

var appLabel string = Title + " " + Version

// -----------------------------------------------------------------

var (
	f   *os.File
	err error

	// from cli flags
	forcedebug      bool = false
	customconfig    bool
	Config          *viper.Viper
	log             zerolog.Logger
	configLocations []string
	home            string
	logBuffer       *lineBuffer
	defaults        map[string]map[string]interface{}

	Map   map[string]interface{}
	Debug bool
)

func init() {
	defaults = make(map[string]map[string]interface{})
	Config = viper.New()
	Config.SetConfigType("toml")
	Config.SetConfigName(Title)
}

type lineBuffer struct {
	lines []string // our buffered log lines
}

func (log *lineBuffer) Write(data []byte) (int, error) {
	line := string(data)
	log.lines = append(log.lines, line)
	return len(data), nil
}

func acquireClue() {
	// define proper console output before we determine a log file location
	log = preLog()

	if home, err = os.UserHomeDir(); err != nil {
		log.Fatal().Err(err).Msg("failed to determine user's home directory, we will not be able to load our configuration if it is stored there!")
	}

	configLocations = append(configLocations, home+"/."+Title+"/")
	configLocations = append(configLocations, "/etc/"+Title+"/")
	configLocations = append(configLocations, "./")
	configLocations = append(configLocations, "../")
	configLocations = append(configLocations, "./.config/")

	// ----------------------------------------------------------------

	log.Debug().Msg("start")

}

/* preLog is to temporarily define pretty printing before we finish initializing our json logger
   note that here we are piping it to our configLog (see: PrintConfigLog)
*/
func preLog() zerolog.Logger {
	if forcedebug {
		zerolog.SetGlobalLevel(zerolog.DebugLevel)
	} else {
		zerolog.SetGlobalLevel(zerolog.InfoLevel)
	}

	// initiate an instance of our custom writer
	logBuffer = new(lineBuffer)
	return zerolog.New(zerolog.ConsoleWriter{Out: logBuffer}).With().Timestamp().Logger()
}

func PrintConfigLog() {
	for _, line := range logBuffer.lines {
		print(line)
	}
}

// Blueprint will initialize our toml configuration engine and define our default configuration values which can be written to a new configuration file if desired
func Blueprint() *viper.Viper {
	for i, arg := range os.Args {
		switch arg {
		case "-c":
			if len(os.Args) <= i-1 {
				panic("syntax error! expected file after -c")
			}

			if f, err = os.Open(os.Args[i+1]); err != nil {
				println("Error opening specified config file: " + os.Args[i+1])
				panic("config file open fatal error: " + err.Error())
			}

			buf, err := ioutil.ReadAll(f)
			err2 := Config.ReadConfig(bytes.NewBuffer(buf))

			if err != nil || err2 != nil {
				panic("Error reading specified config file: " + os.Args[i+1])
				if err != nil {
					panic("config file read fatal error: " + err.Error())
				} else {
					panic("config file read fatal error: " + err2.Error())
				}
			}

			customconfig = true
		}
	}

	if customconfig {
		associate()
		return Config
	}

	acquireClue()

	///////////////////// defaults //////
	//defName := appLabel

	var configSections = []string{"logger", "data", "email"}

	defaults["logger"] = map[string]interface{}{
		"debug":  true,
		"logdir": "./.logs/",
	}
	defaults["data"] = map[string]interface{}{
		"directory":  "./.data/",
		"maxsizeobj": 20,
	}
	defaults["email"] = map[string]interface{}{
		"listen":       "127.0.0.1:25",
		"banner":       "howtouse.email",
		"maxsize_mb":   100,
		"readtimeout":  5,
		"writetimeout": 5,
	}
	for _, def := range configSections {
		Config.SetDefault(def, defaults[def])
	}

	for _, loc := range configLocations {
		Config.AddConfigPath(loc)
	}

	associate()
	return Config
}

func associate() {
	if err = Config.MergeInConfig(); err != nil {
		log.Warn().Err(err).Msg("Writing new config")
		if err = Config.WriteConfigAs("./jupemail.toml"); err != nil {
			panic(err.Error())
		}
	}
	if Debug = Config.GetBool("logger.debug"); Debug {
		println("debug on")
	}
	
}
